#include <pthread.h>
#include <unistd.h>
#include <math.h>
#include "trbsv.h"
#include "tools.h"
#include "trbexport.h"
#include "trberror.h"
#include "svexchange.h"
#include "scenariothr.h"
#include "device.h"
#include "star.h"
#include "success.h"
#include "trbconst.h"

extern DevManager devmanager;
extern Star	star;
extern bool	withDevice;

namespace trbsv {
	bool cycle_condition = true;
	pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
}

void trbsv::processSV(void) {
	std::string instruction;
	bool dialog = trberr::dialog();
	trberr::enableDialog(false);
	while (SVexport::get(instruction) == SUCCESS) {
		int comid;
		switch(SVexchange::translate(instruction, comid)) {
			case SVexchange::IDENT :
				SVexchange::sendIdent(comid);	
				break;

			case SVexchange::STATUS :
				SVexchange::sendStatus(comid);	
				break;

			case SVexchange::DATA :
				SVexchange::sendData(comid);	
				break;

			case SVexchange::QUIT :
				if(!devmanager.isCounting()) {
					devmanager.quitProgram();
				} else {
					SVexchange::setErstatus(SVexchange::BUSY);
					SVexchange::sendError(comid);
				}
				break;

			case SVexchange::INIT :
				if(withDevice && !devmanager.isCounting()) {
					SVexchange::setStatus(SVexchange::BUSY);
					SVexchange::sendOK(comid,INIT_TMOUT);
					if(devmanager.init() != SUCCESS) {
						SVexchange::setErstatus(SVexchange::ERFAT);
						SVexchange::sendError(comid);
					} else {
						SVexchange::setStatus(SVexchange::READY);
						SVexchange::sendOK(comid,0);
					}
				} else {
					SVexchange::setErstatus(SVexchange::BUSY);
					SVexchange::sendError(comid);
				}
				break;

			case SVexchange::PARK :
				if(withDevice && !devmanager.isCounting()) {
					SVexchange::setStatus(SVexchange::BUSY);
					SVexchange::sendOK(comid,PARK_TMOUT);
					devmanager.park();
					SVexchange::setStatus(SVexchange::PARKED);
					SVexchange::sendOK(comid,0);		//PARKED
				} else {
					SVexchange::setErstatus(SVexchange::BUSY);
					SVexchange::sendError(comid);
				}
				break;

			case SVexchange::RUN :
				devmanager.setCurrentScenario(1);
				goto RUN;
			case SVexchange::RUN_BKGR :
				devmanager.setCurrentScenario(2);
RUN:
				if(withDevice && !devmanager.isCounting()) {
					devmanager.setPlayScenarioSVcomid(comid);
					SVexchange::setStatus(SVexchange::BUSY);
					scenariothr::playScenario();
				} else {
					SVexchange::setErstatus(SVexchange::BUSY);
					SVexchange::sendError(comid);
				}
				break;

			case SVexchange::STOP :
				if(withDevice && devmanager.isCounting()) {
					devmanager.setStopScenarioSVcomid(comid);
					SVexchange::sendOK(comid,int(rint(devmanager.getRemainTimeMode())));
					devmanager.stopScenario();
				} else SVexchange::sendOK(comid,0);
				break;

			case SVexchange::STOP_NOW :
				if(withDevice && devmanager.isCounting()) {
					devmanager.setStopCountingSVcomid(comid);
					SVexchange::sendOK(comid,int(rint(devmanager.getCurrentMaxBaseTime())));
					devmanager.stopCounting();
				} else SVexchange::sendOK(comid,0);
				break;

			case SVexchange::NEW_OBJECT :
				if(!devmanager.isCounting()) {
					if(star.setCurrentStar(QString(SVexchange::getNewObject())) == SUCCESS) {
						SVexchange::sendOK(comid,0);
					} else {
						SVexchange::setErstatus(SVexchange::ERANG);
						SVexchange::sendError(comid);
					}
				} else {
					SVexchange::setErstatus(SVexchange::BUSY);
					SVexchange::sendError(comid);
				}
				break;

			default	: break;
		}
	}
	trberr::enableDialog(dialog);
}

int trbsv::sv_cycle(void) {		// execution sv-command

	while(getCycleCondition()) {

		processSV();

//		usleep(ulong(SEC2USEC * devmanager.svPollingTime()));
		usleep(ulong(SEC2USEC * 0.1));
//		jdebug("sv_cycle:next cycle");
	}
//	printf("trbsv::quit sv_cycle\n");
	return 1;
}

void trbsv::setCycleCondition(bool condition) {
	pthread_mutex_lock(&mutex);
	cycle_condition = condition;
	pthread_mutex_unlock(&mutex);
}

bool trbsv::getCycleCondition(void) {
	pthread_mutex_lock(&mutex);
	bool condition = cycle_condition;
	pthread_mutex_unlock(&mutex);
	return condition;
}

void trbsv::quitSVcycle(void) {
	jdebug("trbsv::quitSVcycle()");

	setCycleCondition(false);

	jdebug("end of trbsv::quitSVcycle()");
}
